@@ -1,4 +1,4 @@
-/* Copyright (c)1994-1999 Begemot Computer Associates. All rights reserved.
+/* Copyright (c)1994-2000 Begemot Computer Associates. All rights reserved.
  * See the file COPYRIGHT for details of redistribution and use. */
 
 # include "proc.h"
@@ -6,6 +6,8 @@
 # include <string.h>
 # include <begemot.h>
 
+RCSID("$Id: main.c,v 1.16 2000/03/05 19:39:33 robf Exp $")
+
 Proc	proc;
 char	verbose[128];
 
@@ -70,7 +72,7 @@ name_t	*names;
 
 
 extern IOOps lp_ops, rl_ops, kl_ops, mr_ops, rp_ops, rk_ops;
-extern IOOps tm_ops, qna_ops, proc_ops, memsys_ops;
+extern IOOps tm_ops, qna_ops, proc_ops, memsys_ops, toy_ops;
 
 IOConf ioconf[] = {
 	{ "KDJ11A", &proc_ops, 1 },	/* must be 0 */
@@ -83,12 +85,13 @@ IOConf ioconf[] = {
 	{ "tm", &tm_ops, 0 },
 	{ "qna", &qna_ops, 0 },
 	{ "rk", &rk_ops, 0 },
+	{ "toy", &toy_ops, 0 },
 	{ NULL }
 };
 
 char usgtxt[] =
 "Begemot PDP11 emulator Version %s\n"
-"Copyright (c) 1994-1999 Begemot Computer Associates. All rights reserved.\n"
+"Copyright (c) 1994-2000 Begemot Computer Associates. All rights reserved.\n"
 "\n"
 "Usage: p11 [options] [file]\n"
 "\n"
@@ -191,6 +194,7 @@ main(int argc, char *argv[])
 
 	catch_sigchild();
 
+	init_timer();
 	start_timer();
 	interp();
 
@@ -590,14 +594,14 @@ do_define(char *name, char *val)
 		np = xalloc(sizeof(name_t));
 		np->name = strsave(name);
 		np->val = substitutions(val);
+		np->next = names;
+		names = np;
 		xverb('c', "defining '%s' to '%s'\n", name, np->val);
 	} else {
 		free(np->val);
 		np->val = substitutions(val);
 		xverb('c', "redefining '%s' to '%s'\n", name, np->val);
 	}
-	np->next = names;
-	names = np;
 }
 
 void
@@ -612,7 +616,7 @@ parse_set(char *line)
 		conf_panic("need option name after 'set'");
 
 	value = substitutions(argv[1]);
-	set_option(argv[1], value);
+	set_option(argv[0], value);
 	free(value);
 }
 
@@ -738,8 +742,11 @@ parse_device(char *line)
 	line = substitutions(line);
 	argc = getmfields(line, argv, MAXARGS);
 
-	if(argc == 0)
-		conf_panic("device args");
+	if(argc == 0) {
+		/* ignore lines which have only 'dev' */
+		free(line);
+		return;
+	}
 
 	if(verbose['c']) {
 		xverb('c', "  create device %s(", act_dev->name);
@@ -786,6 +793,7 @@ substitutions(char *line)
 	int c;
 	char name[MAXNAME];
 	int namelen;
+	int qflag;
 
 	if(line == NULL)
 		return strsave("");
@@ -826,27 +834,40 @@ substitutions(char *line)
 		 * Collect the name
 		 */
 		namelen = 0;
-		while((c = nextc()) != EOF && c != ')') {
-			if(!isascii(c) || (!isalnum(c) && c != '_'))
-				conf_panic("illegal char in name '0x%02x'", c);
+		while((c = nextc()) != EOF && isascii(c) && (isalnum(c) || c == '_')) {
 			if(namelen == MAXNAME - 1)
 				conf_panic("name too long");
 			name[namelen++] = c;
 		}
+		name[namelen] = 0;
+
+		/*
+		 * Collect flags
+		 */
+		qflag = 0;
+		if(c == '?') {
+			qflag = 1;
+			c = nextc();
+		}
 		if(c == EOF)
 			conf_panic("EOF in substitution");
-		name[namelen] = 0;
+		if(!isascii(c) || c != ')')
+			conf_panic("illegal char in name '0x%02x'", c);
+
 		check_name(name);
 		for(np = names; np != NULL; np = np->next)
 			if(strcmp(np->name, name) == 0)
 				break;
-		if(np == NULL)
-			conf_panic("undefined name '%s'", name);
-
-		pushstr(np->val);
+		if(qflag) {
+			if(np != NULL)
+				pushstr(np->val);
+		} else {
+			if(np == NULL)
+				conf_panic("undefined name '%s'", name);
+			pushstr(np->val);
+		}
 	}
 	STUFF_C(0);
-
 	return nline;
 }
 
